@@ -42,11 +42,12 @@ int font_width(struct font *font, const char *string);
  * @text: text to be generated as texture
  * @font: font description
  * @color: foreground color in rgba format
+ * @bg_color: background color in rgba format
  * @arrow: arrow (utf8) character to show or NULL for none
  */
 void font_buffer_create(struct lab_data_buffer **buffer, int max_width,
-	const char *text, struct font *font, float *color, const char *arrow,
-	double scale);
+	const char *text, struct font *font, const float *color,
+	const float *bg_color, const char *arrow, double scale);
 
 /**
  * font_finish - free some font related resources
