@@ -238,6 +238,10 @@ struct view {
 		struct wlr_scene_rect *background;
 		struct scaled_font_buffer *text;
 	} resize_indicator;
+	struct resize_outlines {
+		struct wlr_box view_geo;
+		struct multi_rect *rect;
+	} resize_outlines;
 
 	struct foreign_toplevel {
 		struct wlr_foreign_toplevel_handle_v1 *handle;
@@ -266,6 +270,9 @@ struct view_query {
 	struct wl_list link;
 	char *identifier;
 	char *title;
+	int window_type;
+	char *sandbox_engine;
+	char *sandbox_app_id;
 };
 
 struct xdg_toplevel_view {
@@ -304,6 +311,13 @@ enum lab_view_criteria {
  */
 struct view *view_from_wlr_surface(struct wlr_surface *surface);
 
+/**
+ * view_query_create() - Create a new heap allocated view query with
+ * all members initialized to their default values (window_type = -1,
+ * NULL for strings)
+ */
+struct view_query *view_query_create(void);
+
 /**
  * view_query_free() - Free a given view query
  * @query: Query to be freed.
@@ -455,10 +469,13 @@ int view_effective_height(struct view *view, bool use_pending);
 void view_center(struct view *view, const struct wlr_box *ref);
 
 /**
- * view_place_initial - apply initial placement strategy to view
+ * view_place_by_policy - apply placement strategy to view
  * @view: view to be placed
+ * @allow_cursor: set to false to ignore center-on-cursor policy
+ * @policy: placement policy to apply
  */
-void view_place_initial(struct view *view, bool allow_cursor);
+void view_place_by_policy(struct view *view, bool allow_cursor,
+	enum view_placement_policy);
 void view_constrain_size_to_that_of_usable_area(struct view *view);
 
 void view_restore_to(struct view *view, struct wlr_box geometry);
@@ -467,6 +484,7 @@ void view_maximize(struct view *view, enum view_axis axis,
 	bool store_natural_geometry);
 void view_set_fullscreen(struct view *view, bool fullscreen);
 void view_toggle_maximize(struct view *view, enum view_axis axis);
+bool view_wants_decorations(struct view *view);
 void view_toggle_decorations(struct view *view);
 
 bool view_is_always_on_top(struct view *view);
@@ -477,9 +495,12 @@ void view_toggle_always_on_bottom(struct view *view);
 void view_toggle_visible_on_all_workspaces(struct view *view);
 
 bool view_is_tiled(struct view *view);
+bool view_is_tiled_and_notify_tiled(struct view *view);
 bool view_is_floating(struct view *view);
 void view_move_to_workspace(struct view *view, struct workspace *workspace);
-void view_set_decorations(struct view *view, bool decorations);
+enum ssd_mode view_get_ssd_mode(struct view *view);
+void view_set_ssd_mode(struct view *view, enum ssd_mode mode);
+void view_set_decorations(struct view *view, enum ssd_mode mode, bool force_ssd);
 void view_toggle_fullscreen(struct view *view);
 void view_invalidate_last_layout_geometry(struct view *view);
 void view_adjust_for_layout_change(struct view *view);
@@ -524,6 +545,7 @@ struct output *view_get_adjacent_output(struct view *view, enum view_edge edge,
 	bool wrap);
 enum view_axis view_axis_parse(const char *direction);
 enum view_edge view_edge_parse(const char *direction);
+enum view_placement_policy view_placement_parse(const char *policy);
 
 /* xdg.c */
 struct wlr_xdg_surface *xdg_surface_from_view(struct view *view);
