@@ -33,7 +33,7 @@ multi_rect_create(struct wlr_scene_tree *parent, float *colors[3], int line_widt
 		wlr_scene_node_set_position(&rect->top[i]->node,
 			i * line_width, i * line_width);
 		wlr_scene_node_set_position(&rect->left[i]->node,
-			i * line_width, i * line_width);
+			i * line_width, (i + 1) * line_width);
 	}
 	return rect;
 }
@@ -44,10 +44,22 @@ multi_rect_set_size(struct multi_rect *rect, int width, int height)
 	assert(rect);
 	int line_width = rect->line_width;
 
+	/*
+	 * The outmost outline is drawn like below:
+	 *
+	 * |--width--|
+	 *
+	 * +---------+  ---
+	 * +-+-----+-+   |
+	 * | |     | | height
+	 * | |     | |   |
+	 * +-+-----+-+   |
+	 * +---------+  ---
+	 */
 	for (size_t i = 0; i < 3; i++) {
 		/* Reposition, top and left don't ever change */
 		wlr_scene_node_set_position(&rect->right[i]->node,
-			width - (i + 1) * line_width, i * line_width);
+			width - (i + 1) * line_width, (i + 1) * line_width);
 		wlr_scene_node_set_position(&rect->bottom[i]->node,
 			i * line_width, height - (i + 1) * line_width);
 
@@ -57,9 +69,9 @@ multi_rect_set_size(struct multi_rect *rect, int width, int height)
 		wlr_scene_rect_set_size(rect->bottom[i],
 			width - i * line_width * 2, line_width);
 		wlr_scene_rect_set_size(rect->left[i],
-			line_width, height - i * line_width * 2);
+			line_width, height - (i + 1) * line_width * 2);
 		wlr_scene_rect_set_size(rect->right[i],
-			line_width, height - i * line_width * 2);
+			line_width, height - (i + 1) * line_width * 2);
 	}
 }
 
@@ -83,9 +95,21 @@ draw_cairo_border(cairo_t *cairo, struct wlr_fbox fbox, double line_width)
 
 /* Sets the cairo color. Splits the single color channels */
 void
-set_cairo_color(cairo_t *cairo, float *c)
+set_cairo_color(cairo_t *cairo, const float *c)
 {
-	cairo_set_source_rgba(cairo, c[0], c[1], c[2], c[3]);
+	/*
+	 * We are dealing with pre-multiplied colors
+	 * but cairo expects unmultiplied colors here
+	 */
+	float alpha = c[3];
+
+	if (alpha == 0.0f) {
+		cairo_set_source_rgba(cairo, 0, 0, 0, 0);
+		return;
+	}
+
+	cairo_set_source_rgba(cairo, c[0] / alpha, c[1] / alpha,
+		c[2] / alpha, alpha);
 }
 
 struct surface_context
