@@ -229,17 +229,23 @@ desktop_update_top_layer_visiblity(struct server *server)
 		wlr_scene_node_set_enabled(&output->layer_tree[top]->node, true);
 	}
 
-	/* And disable them again when there is a view in fullscreen */
-	enum lab_view_criteria criteria =
-		LAB_VIEW_CRITERIA_CURRENT_WORKSPACE | LAB_VIEW_CRITERIA_FULLSCREEN;
-	for_each_view(view, &server->views, criteria) {
+	/*
+	 * And disable them again when there is a fullscreen view without
+	 * any views above it
+	 */
+	uint64_t outputs_covered = 0;
+	for_each_view(view, &server->views, LAB_VIEW_CRITERIA_CURRENT_WORKSPACE) {
 		if (view->minimized) {
 			continue;
 		}
 		if (!output_is_usable(view->output)) {
 			continue;
 		}
-		wlr_scene_node_set_enabled(&view->output->layer_tree[top]->node, false);
+		if (view->fullscreen && !(view->outputs & outputs_covered)) {
+			wlr_scene_node_set_enabled(
+				&view->output->layer_tree[top]->node, false);
+		}
+		outputs_covered |= view->outputs;
 	}
 }
 
@@ -330,6 +336,7 @@ get_cursor_context(struct server *server)
 				ret.type = LAB_SSD_CLIENT;
 				ret.surface = get_surface_from_layer_node(node);
 				return ret;
+			case LAB_NODE_DESC_SESSION_LOCK_SURFACE: /* fallthrough */
 			case LAB_NODE_DESC_IME_POPUP:
 				ret.type = LAB_SSD_CLIENT;
 				ret.surface = lab_wlr_surface_from_node(ret.node);
