@@ -3,7 +3,7 @@
 #include "input/keyboard.h"
 #include "labwc.h"
 #include "regions.h"
-#include "resize_indicator.h"
+#include "resize-indicator.h"
 #include "snap.h"
 #include "view.h"
 #include "window-rules.h"
