@@ -356,6 +356,7 @@ popup_handle_destroy(struct wl_listener *listener, void *data)
 		wl_container_of(listener, popup, destroy);
 	wl_list_remove(&popup->destroy.link);
 	wl_list_remove(&popup->new_popup.link);
+	wl_list_remove(&popup->reposition.link);
 
 	/* Usually already removed unless there was no commit at all */
 	if (popup->commit.notify) {
@@ -381,6 +382,15 @@ popup_handle_commit(struct wl_listener *listener, void *data)
 	}
 }
 
+static void
+popup_handle_reposition(struct wl_listener *listener, void *data)
+{
+	struct lab_layer_popup *popup =
+		wl_container_of(listener, popup, reposition);
+	wlr_xdg_popup_unconstrain_from_box(popup->wlr_popup,
+		&popup->output_toplevel_sx_box);
+}
+
 static void popup_handle_new_popup(struct wl_listener *listener, void *data);
 
 static struct lab_layer_popup *
@@ -410,6 +420,9 @@ create_popup(struct wlr_xdg_popup *wlr_popup, struct wlr_scene_tree *parent)
 	popup->commit.notify = popup_handle_commit;
 	wl_signal_add(&wlr_popup->base->surface->events.commit, &popup->commit);
 
+	popup->reposition.notify = popup_handle_reposition;
+	wl_signal_add(&wlr_popup->events.reposition, &popup->reposition);
+
 	return popup;
 }
 
