@@ -108,15 +108,14 @@ resistance_move_apply(struct view *view, double *x, double *y)
 
 	if (rc.screen_edge_strength != 0) {
 		/* Find any relevant output edges encountered by this move */
-		edges_find_outputs(&next_edges, view, target, NULL,
-			check_edge_output, /* use_pending */ false);
+		edges_find_outputs(&next_edges, view,
+			view->current, target, NULL, check_edge_output);
 	}
 
 	if (rc.window_edge_strength != 0) {
 		/* Find any relevant window edges encountered by this move */
-		edges_find_neighbors(&next_edges,
-			view, target, NULL, check_edge_window,
-			/* use_pending */ false, /* ignore_hidden */ true);
+		edges_find_neighbors(&next_edges, view, view->current, target,
+			NULL, check_edge_window, /* ignore_hidden */ true);
 	}
 
 	/* If any "best" edges were encountered during this move, snap motion */
@@ -138,15 +137,14 @@ resistance_resize_apply(struct view *view, struct wlr_box *new_geom)
 
 	if (rc.screen_edge_strength != 0) {
 		/* Find any relevant output edges encountered by this move */
-		edges_find_outputs(&next_edges, view, *new_geom, NULL,
-			check_edge_output, /* use_pending */ false);
+		edges_find_outputs(&next_edges, view,
+			view->current, *new_geom, NULL, check_edge_output);
 	}
 
 	if (rc.window_edge_strength != 0) {
 		/* Find any relevant window edges encountered by this move */
-		edges_find_neighbors(&next_edges,
-			view, *new_geom, NULL, check_edge_window,
-			/* use_pending */ false, /* ignore_hidden */ true);
+		edges_find_neighbors(&next_edges, view, view->current, *new_geom,
+			NULL, check_edge_window, /* ignore_hidden */ true);
 	}
 
 	/* If any "best" edges were encountered during this move, snap motion */
