@@ -66,6 +66,8 @@ struct rounded_corner_ctx {
 	enum corner corner;
 };
 
+#define zero_array(arr) memset(arr, 0, sizeof(arr))
+
 static struct lab_data_buffer *rounded_rect(struct rounded_corner_ctx *ctx);
 
 static void
@@ -533,8 +535,7 @@ theme_builtin(struct theme *theme, struct server *server)
 
 	/* inherit settings in post_processing() if not set elsewhere */
 	theme->osd_window_switcher_preview_border_width = INT_MIN;
-	memset(theme->osd_window_switcher_preview_border_color, 0,
-		sizeof(theme->osd_window_switcher_preview_border_color));
+	zero_array(theme->osd_window_switcher_preview_border_color);
 	theme->osd_window_switcher_preview_border_color[0][0] = FLT_MIN;
 
 	theme->osd_workspace_switcher_boxes_width = 20;
@@ -565,12 +566,14 @@ theme_builtin(struct theme *theme, struct server *server)
 	/* inherit settings in post_processing() if not set elsewhere */
 	theme->snapping_overlay_region.border_width = INT_MIN;
 	theme->snapping_overlay_edge.border_width = INT_MIN;
-	memset(theme->snapping_overlay_region.border_color, 0,
-		sizeof(theme->snapping_overlay_region.border_color));
+	zero_array(theme->snapping_overlay_region.border_color);
 	theme->snapping_overlay_region.border_color[0][0] = FLT_MIN;
-	memset(theme->snapping_overlay_edge.border_color, 0,
-		sizeof(theme->snapping_overlay_edge.border_color));
+	zero_array(theme->snapping_overlay_edge.border_color);
 	theme->snapping_overlay_edge.border_color[0][0] = FLT_MIN;
+
+	/* magnifier */
+	parse_hexstr("#ff0000", theme->mag_border_color);
+	theme->mag_border_width = 1;
 }
 
 static void
@@ -826,6 +829,13 @@ entry(struct theme *theme, const char *key, const char *value)
 	if (match_glob(key, "snapping.overlay.edge.border.color")) {
 		parse_hexstrs(value, theme->snapping_overlay_edge.border_color);
 	}
+
+	if (match_glob(key, "magnifier.border.width")) {
+		theme->mag_border_width = atoi(value);
+	}
+	if (match_glob(key, "magnifier.border.color")) {
+		parse_hexstr(value, theme->mag_border_color);
+	}
 }
 
 static void
@@ -1387,10 +1397,35 @@ theme_init(struct theme *theme, struct server *server, const char *theme_name)
 void
 theme_finish(struct theme *theme)
 {
+	zdrop(&theme->button_close_active_unpressed);
+	zdrop(&theme->button_maximize_active_unpressed);
+	zdrop(&theme->button_restore_active_unpressed);
+	zdrop(&theme->button_iconify_active_unpressed);
+	zdrop(&theme->button_menu_active_unpressed);
+
+	zdrop(&theme->button_close_inactive_unpressed);
+	zdrop(&theme->button_maximize_inactive_unpressed);
+	zdrop(&theme->button_restore_inactive_unpressed);
+	zdrop(&theme->button_iconify_inactive_unpressed);
+	zdrop(&theme->button_menu_inactive_unpressed);
+
+	zdrop(&theme->button_close_active_hover);
+	zdrop(&theme->button_maximize_active_hover);
+	zdrop(&theme->button_restore_active_hover);
+	zdrop(&theme->button_iconify_active_hover);
+	zdrop(&theme->button_menu_active_hover);
+
+	zdrop(&theme->button_close_inactive_hover);
+	zdrop(&theme->button_maximize_inactive_hover);
+	zdrop(&theme->button_restore_inactive_hover);
+	zdrop(&theme->button_iconify_inactive_hover);
+	zdrop(&theme->button_menu_inactive_hover);
+
 	zdrop(&theme->corner_top_left_active_normal);
 	zdrop(&theme->corner_top_left_inactive_normal);
 	zdrop(&theme->corner_top_right_active_normal);
 	zdrop(&theme->corner_top_right_inactive_normal);
+
 	zdrop(&theme->shadow_corner_top_active);
 	zdrop(&theme->shadow_corner_bottom_active);
 	zdrop(&theme->shadow_edge_active);
