@@ -46,6 +46,12 @@ view_impl_map(struct view *view)
 		}
 	}
 
+	/*
+	 * Some clients (e.g. Steam's Big Picture Mode window) request
+	 * fullscreen before mapping.
+	 */
+	desktop_update_top_layer_visiblity(view->server);
+
 	wlr_log(WLR_DEBUG, "[map] identifier=%s, title=%s\n",
 		view_get_string_prop(view, "app_id"),
 		view_get_string_prop(view, "title"));
