@@ -3,6 +3,7 @@
 #include <stdio.h>
 #include <strings.h>
 #include <wlr/types/wlr_output_layout.h>
+#include <wlr/types/wlr_security_context_v1.h>
 #include "common/macros.h"
 #include "common/match.h"
 #include "common/mem.h"
@@ -13,7 +14,7 @@
 #include "osd.h"
 #include "placement.h"
 #include "regions.h"
-#include "resize_indicator.h"
+#include "resize-indicator.h"
 #include "snap-constraints.h"
 #include "snap.h"
 #include "ssd.h"
@@ -53,12 +54,33 @@ view_from_wlr_surface(struct wlr_surface *surface)
 	return NULL;
 }
 
+static const struct wlr_security_context_v1_state *
+security_context_from_view(struct view *view)
+{
+	if (view && view->surface && view->surface->resource) {
+		struct wl_client *client = wl_resource_get_client(view->surface->resource);
+		return wlr_security_context_manager_v1_lookup_client(
+			view->server->security_context_manager_v1, client);
+	}
+	return NULL;
+}
+
+struct view_query *
+view_query_create(void)
+{
+	struct view_query *query = znew(*query);
+	query->window_type = -1;
+	return query;
+}
+
 void
 view_query_free(struct view_query *query)
 {
 	wl_list_remove(&query->link);
 	free(query->identifier);
 	free(query->title);
+	free(query->sandbox_engine);
+	free(query->sandbox_app_id);
 	free(query);
 }
 
@@ -71,13 +93,34 @@ view_matches_query(struct view *view, struct view_query *query)
 	const char *identifier = view_get_string_prop(view, "app_id");
 	if (match && query->identifier) {
 		empty = false;
-		match &= match_glob(query->identifier, identifier);
+		match &= identifier && match_glob(query->identifier, identifier);
 	}
 
 	const char *title = view_get_string_prop(view, "title");
 	if (match && query->title) {
 		empty = false;
-		match &= match_glob(query->title, title);
+		match &= title && match_glob(query->title, title);
+	}
+
+	if (match && query->window_type >= 0) {
+		empty = false;
+		match &= view_contains_window_type(view, query->window_type);
+	}
+
+	if (match && query->sandbox_engine) {
+		const struct wlr_security_context_v1_state *security_context =
+			security_context_from_view(view);
+		empty = false;
+		match &= security_context && security_context->sandbox_engine
+			&& match_glob(query->sandbox_engine, security_context->sandbox_engine);
+	}
+
+	if (match && query->sandbox_app_id) {
+		const struct wlr_security_context_v1_state *security_context =
+			security_context_from_view(view);
+		empty = false;
+		match &= security_context && security_context->app_id
+			&& match_glob(query->sandbox_app_id, security_context->app_id);
 	}
 
 	return !empty && match;
@@ -387,16 +430,20 @@ view_update_outputs(struct view *view)
 	struct output *output;
 	struct wlr_output_layout *layout = view->server->output_layout;
 
-	view->outputs = 0;
+	uint64_t new_outputs = 0;
 	wl_list_for_each(output, &view->server->outputs, link) {
 		if (output_is_usable(output) && wlr_output_layout_intersects(
 				layout, output->wlr_output, &view->current)) {
-			view->outputs |= (1ull << output->scene_output->index);
+			new_outputs |= (1ull << output->scene_output->index);
 		}
 	}
 
-	if (view->toplevel.handle) {
-		foreign_toplevel_update_outputs(view);
+	if (new_outputs != view->outputs) {
+		view->outputs = new_outputs;
+		if (view->toplevel.handle) {
+			foreign_toplevel_update_outputs(view);
+		}
+		desktop_update_top_layer_visiblity(view->server);
 	}
 }
 
@@ -681,12 +728,16 @@ view_compute_centered_position(struct view *view, const struct wlr_box *ref,
 	*x = ref->x + (ref->width - width) / 2;
 	*y = ref->y + (ref->height - height) / 2;
 
-	/* If view is bigger than usable area, just top/left align it */
+	/* Fit the view within the usable area */
 	if (*x < usable.x) {
 		*x = usable.x;
+	} else if (*x + width > usable.x + usable.width) {
+		*x = usable.x + usable.width - width;
 	}
 	if (*y < usable.y) {
 		*y = usable.y;
+	} else if (*y + height > usable.y + usable.height) {
+		*y = usable.y + usable.height - height;
 	}
 
 	*x += margin.left;
@@ -790,10 +841,12 @@ view_store_natural_geometry(struct view *view)
 
 	/**
 	 * If an application was started maximized or fullscreened, its
-	 * natural_geometry width/height may still be zero in which case we set
-	 * some fallback values. This is the case with foot and Qt applications.
+	 * natural_geometry width/height may still be zero (or very small
+	 * values) in which case we set some fallback values. This is the case
+	 * with foot and some Qt/Tk applications.
 	 */
-	if (wlr_box_empty(&view->pending)) {
+	if (view->pending.width < LAB_MIN_VIEW_WIDTH
+			|| view->pending.height < LAB_MIN_VIEW_HEIGHT) {
 		set_fallback_geometry(view);
 	} else {
 		view->natural_geometry = view->pending;
@@ -824,12 +877,13 @@ view_center(struct view *view, const struct wlr_box *ref)
 }
 
 void
-view_place_initial(struct view *view, bool allow_cursor)
+view_place_by_policy(struct view *view, bool allow_cursor,
+		enum view_placement_policy policy)
 {
-	if (allow_cursor && rc.placement_policy == LAB_PLACE_CURSOR) {
+	if (allow_cursor && policy == LAB_PLACE_CURSOR) {
 		view_move_to_cursor(view);
 		return;
-	} else if (rc.placement_policy == LAB_PLACE_AUTOMATIC) {
+	} else if (policy == LAB_PLACE_AUTOMATIC) {
 		struct wlr_box geometry = view->pending;
 		if (placement_find_best(view, &geometry)) {
 			view_move(view, geometry.x, geometry.y);
@@ -1119,6 +1173,23 @@ view_is_tiled(struct view *view)
 		|| view->tiled_region_evacuate);
 }
 
+bool
+view_is_tiled_and_notify_tiled(struct view *view)
+{
+	switch (rc.snap_tiling_events_mode) {
+	case LAB_TILING_EVENTS_NEVER:
+		return false;
+	case LAB_TILING_EVENTS_REGION:
+		return view->tiled_region || view->tiled_region_evacuate;
+	case LAB_TILING_EVENTS_EDGE:
+		return view->tiled;
+	case LAB_TILING_EVENTS_ALWAYS:
+		return view_is_tiled(view);
+	}
+
+	return false;
+}
+
 bool
 view_is_floating(struct view *view)
 {
@@ -1208,30 +1279,61 @@ view_toggle_maximize(struct view *view, enum view_axis axis)
 	}
 }
 
+bool
+view_wants_decorations(struct view *view)
+{
+	/* Window-rules take priority if they exist for this view */
+	switch (window_rules_get_property(view, "serverDecoration")) {
+	case LAB_PROP_TRUE:
+		return true;
+	case LAB_PROP_FALSE:
+		return false;
+	default:
+		break;
+	}
+
+	/*
+	 * view->ssd_preference may be set by the decoration implementation
+	 * e.g. src/decorations/xdg-deco.c or src/decorations/kde-deco.c.
+	 */
+	switch (view->ssd_preference) {
+	case LAB_SSD_PREF_SERVER:
+		return true;
+	case LAB_SSD_PREF_CLIENT:
+		return false;
+	default:
+		/*
+		 * We don't know anything about the client preference
+		 * so fall back to core.decoration settings in rc.xml
+		 */
+		return rc.xdg_shell_server_side_deco;
+	}
+}
+
+void
+view_set_decorations(struct view *view, enum ssd_mode mode, bool force_ssd)
+{
+	assert(view);
+
+	if (force_ssd || view_wants_decorations(view)
+			|| mode < view_get_ssd_mode(view)) {
+		view_set_ssd_mode(view, mode);
+	}
+}
+
 void
 view_toggle_decorations(struct view *view)
 {
 	assert(view);
 
-	/* Reject decoration toggles when shaded */
-	if (view->shaded) {
-		return;
+	enum ssd_mode mode = view_get_ssd_mode(view);
+	if (rc.ssd_keep_border && mode == LAB_SSD_MODE_FULL) {
+		view_set_ssd_mode(view, LAB_SSD_MODE_BORDER);
+	} else if (mode != LAB_SSD_MODE_NONE) {
+		view_set_ssd_mode(view, LAB_SSD_MODE_NONE);
+	} else {
+		view_set_ssd_mode(view, LAB_SSD_MODE_FULL);
 	}
-
-	if (rc.ssd_keep_border && view->ssd_enabled && view->ssd
-			&& !view->ssd_titlebar_hidden) {
-		/*
-		 * ssd_titlebar_hidden has to be set before calling
-		 * ssd_titlebar_hide() to make ssd_thickness() happy.
-		 */
-		view->ssd_titlebar_hidden = true;
-		ssd_titlebar_hide(view->ssd);
-		if (!view_is_floating(view)) {
-			view_apply_special_geometry(view);
-		}
-		return;
-	}
-	view_set_decorations(view, !view->ssd_enabled);
 }
 
 bool
@@ -1313,26 +1415,46 @@ undecorate(struct view *view)
 	view->ssd = NULL;
 }
 
-void
-view_set_decorations(struct view *view, bool decorations)
+enum ssd_mode
+view_get_ssd_mode(struct view *view)
 {
 	assert(view);
 
-	if (view->ssd_enabled != decorations && !view->fullscreen) {
-		/*
-		 * Set view->ssd_enabled first since it is referenced
-		 * within the call tree of ssd_create()
-		 */
-		view->ssd_enabled = decorations;
-		if (decorations) {
-			decorate(view);
-		} else {
-			undecorate(view);
-			view->ssd_titlebar_hidden = false;
-		}
-		if (!view_is_floating(view)) {
-			view_apply_special_geometry(view);
-		}
+	if (!view->ssd_enabled) {
+		return LAB_SSD_MODE_NONE;
+	} else if (view->ssd_titlebar_hidden) {
+		return LAB_SSD_MODE_BORDER;
+	} else {
+		return LAB_SSD_MODE_FULL;
+	}
+}
+
+void
+view_set_ssd_mode(struct view *view, enum ssd_mode mode)
+{
+	assert(view);
+
+	if (view->shaded || view->fullscreen
+			|| mode == view_get_ssd_mode(view)) {
+		return;
+	}
+
+	/*
+	 * Set these first since they are referenced
+	 * within the call tree of ssd_create() and ssd_thickness()
+	 */
+	view->ssd_enabled = mode != LAB_SSD_MODE_NONE;
+	view->ssd_titlebar_hidden = mode != LAB_SSD_MODE_FULL;
+
+	if (view->ssd_enabled) {
+		decorate(view);
+		ssd_set_titlebar(view->ssd, !view->ssd_titlebar_hidden);
+	} else {
+		undecorate(view);
+	}
+
+	if (!view_is_floating(view)) {
+		view_apply_special_geometry(view);
 	}
 }
 
@@ -1857,6 +1979,24 @@ view_edge_parse(const char *direction)
 	}
 }
 
+enum view_placement_policy
+view_placement_parse(const char *policy)
+{
+	if (!policy) {
+		return LAB_PLACE_CENTER;
+	}
+
+	if (!strcasecmp(policy, "automatic")) {
+		return LAB_PLACE_AUTOMATIC;
+	} else if (!strcasecmp(policy, "cursor")) {
+		return LAB_PLACE_CURSOR;
+	} else if (!strcasecmp(policy, "center")) {
+		return LAB_PLACE_CENTER;
+	}
+
+	return LAB_PLACE_INVALID;
+}
+
 void
 view_snap_to_edge(struct view *view, enum view_edge edge,
 			bool across_outputs, bool store_natural_geometry)
@@ -1960,7 +2100,8 @@ view_move_to_output(struct view *view, struct output *output)
 		struct wlr_box output_area = output_usable_area_in_layout_coords(output);
 		view->pending.x = output_area.x;
 		view->pending.y = output_area.y;
-		view_place_initial(view, /* allow_cursor */ false);
+		view_place_by_policy(view,
+				/* allow_cursor */ false, rc.placement_policy);
 	} else if (view->maximized != VIEW_AXIS_NONE) {
 		view_apply_maximized_geometry(view);
 	} else if (view->tiled) {
@@ -2037,6 +2178,7 @@ view_move_to_front(struct view *view)
 	}
 
 	cursor_update_focus(view->server);
+	desktop_update_top_layer_visiblity(view->server);
 }
 
 void
@@ -2050,6 +2192,7 @@ view_move_to_back(struct view *view)
 	move_to_back(root);
 
 	cursor_update_focus(view->server);
+	desktop_update_top_layer_visiblity(view->server);
 }
 
 struct view *
@@ -2253,6 +2396,10 @@ view_destroy(struct view *view)
 		server->active_view = NULL;
 	}
 
+	if (server->session_lock_manager->last_active_view == view) {
+		server->session_lock_manager->last_active_view = NULL;
+	}
+
 	if (server->last_raised_view == view) {
 		server->last_raised_view = NULL;
 	}
