@@ -14,7 +14,7 @@
 #include "window-rules.h"
 #include "workspaces.h"
 
-#define LAB_XDG_SHELL_VERSION (2)
+#define LAB_XDG_SHELL_VERSION (3)
 #define CONFIGURE_TIMEOUT_MS 100
 
 static struct xdg_toplevel_view *
@@ -75,37 +75,6 @@ handle_new_popup(struct wl_listener *listener, void *data)
 	xdg_popup_create(view, wlr_popup);
 }
 
-static bool
-has_ssd(struct view *view)
-{
-	/* Window-rules take priority if they exist for this view */
-	switch (window_rules_get_property(view, "serverDecoration")) {
-	case LAB_PROP_TRUE:
-		return true;
-	case LAB_PROP_FALSE:
-		return false;
-	default:
-		break;
-	}
-
-	/*
-	 * view->ssd_preference may be set by the decoration implementation
-	 * e.g. src/decorations/xdg-deco.c or src/decorations/kde-deco.c.
-	 */
-	switch (view->ssd_preference) {
-	case LAB_SSD_PREF_SERVER:
-		return true;
-	case LAB_SSD_PREF_CLIENT:
-		return false;
-	default:
-		/*
-		 * We don't know anything about the client preference
-		 * so fall back to core.decoration settings in rc.xml
-		 */
-		return rc.xdg_shell_server_side_deco;
-	}
-}
-
 static void
 handle_commit(struct wl_listener *listener, void *data)
 {
@@ -530,7 +499,7 @@ set_initial_position(struct view *view)
 	}
 
 	/* All other views are placed according to a configured strategy */
-	view_place_initial(view, /* allow_cursor */ true);
+	view_place_by_policy(view, /* allow_cursor */ true, rc.placement_policy);
 }
 
 static const char *
@@ -599,7 +568,11 @@ xdg_toplevel_view_map(struct view *view)
 
 		init_foreign_toplevel(view);
 
-		view_set_decorations(view, has_ssd(view));
+		if (view_wants_decorations(view)) {
+			view_set_ssd_mode(view, LAB_SSD_MODE_FULL);
+		} else {
+			view_set_ssd_mode(view, LAB_SSD_MODE_NONE);
+		}
 
 		/*
 		 * Set initial "pending" dimensions (may be modified by
