@@ -6,6 +6,7 @@
 #include "common/array.h"
 #include "common/macros.h"
 #include "common/mem.h"
+#include "config/rcxml.h"
 #include "labwc.h"
 #include "node.h"
 #include "ssd.h"
@@ -435,8 +436,11 @@ handle_request_maximize(struct wl_listener *listener, void *data)
 		 * Set decorations early to avoid changing geometry
 		 * after maximize (reduces visual glitches).
 		 */
-		view_set_decorations(view,
-			want_deco(xwayland_surface_from_view(view)));
+		if (want_deco(xwayland_surface_from_view(view))) {
+			view_set_ssd_mode(view, LAB_SSD_MODE_FULL);
+		} else {
+			view_set_ssd_mode(view, LAB_SSD_MODE_NONE);
+		}
 	}
 	view_toggle_maximize(view, VIEW_AXIS_BOTH);
 }
@@ -508,7 +512,11 @@ handle_set_decorations(struct wl_listener *listener, void *data)
 		wl_container_of(listener, xwayland_view, set_decorations);
 	struct view *view = &xwayland_view->base;
 
-	view_set_decorations(view, want_deco(xwayland_view->xwayland_surface));
+	if (want_deco(xwayland_view->xwayland_surface)) {
+		view_set_ssd_mode(view, LAB_SSD_MODE_FULL);
+	} else {
+		view_set_ssd_mode(view, LAB_SSD_MODE_NONE);
+	}
 }
 
 static void
@@ -571,7 +579,8 @@ set_initial_position(struct view *view,
 		view_constrain_size_to_that_of_usable_area(view);
 
 		if (view_is_floating(view)) {
-			view_place_initial(view, /* allow_cursor */ true);
+			view_place_by_policy(view,
+				/* allow_cursor */ true, rc.placement_policy);
 		} else {
 			/*
 			 * View is maximized/fullscreen. Center the
@@ -658,7 +667,11 @@ xwayland_view_map(struct view *view)
 	 */
 	view_set_fullscreen(view, xwayland_surface->fullscreen);
 	if (!view->been_mapped) {
-		view_set_decorations(view, want_deco(xwayland_surface));
+		if (want_deco(xwayland_surface)) {
+			view_set_ssd_mode(view, LAB_SSD_MODE_FULL);
+		} else {
+			view_set_ssd_mode(view, LAB_SSD_MODE_NONE);
+		}
 	}
 	enum view_axis axis = VIEW_AXIS_NONE;
 	if (xwayland_surface->maximized_horz) {
@@ -1040,7 +1053,8 @@ void
 xwayland_server_init(struct server *server, struct wlr_compositor *compositor)
 {
 	server->xwayland =
-		wlr_xwayland_create(server->wl_display, compositor, true);
+		wlr_xwayland_create(server->wl_display,
+			compositor, /* lazy */ !rc.xwayland_persistence);
 	if (!server->xwayland) {
 		wlr_log(WLR_ERROR, "cannot create xwayland server");
 		exit(EXIT_FAILURE);
@@ -1116,6 +1130,56 @@ xwayland_adjust_stacking_order(struct server *server)
 	wl_array_release(&views);
 }
 
+void
+xwayland_reset_cursor(struct server *server)
+{
+	/*
+	 * As xwayland caches the pixel data when not yet started up
+	 * due to the delayed lazy startup approach, we do have to
+	 * re-set the xwayland cursor image. Otherwise the first X11
+	 * client connected will cause the xwayland server to use
+	 * the cached (and potentially destroyed) pixel data.
+	 *
+	 * Calling this function after reloading the cursor theme
+	 * ensures that the cached pixel data keeps being valid.
+	 *
+	 * To reproduce:
+	 * - Compile with b_sanitize=address,undefined
+	 * - Start labwc (nothing in autostart that could create
+	 *   a X11 connection, e.g. no GTK or X11 application)
+	 * - Reconfigure
+	 * - Start some X11 client
+	 */
+
+	if (!server->xwayland) {
+		return;
+	}
+
+	struct wlr_xcursor *xcursor = wlr_xcursor_manager_get_xcursor(
+		server->seat.xcursor_manager, XCURSOR_DEFAULT, 1);
+
+	if (xcursor && !server->xwayland->xwm) {
+		/* Prevents setting the cursor on an active xwayland server */
+		struct wlr_xcursor_image *image = xcursor->images[0];
+		wlr_xwayland_set_cursor(server->xwayland, image->buffer,
+			image->width * 4, image->width,
+			image->height, image->hotspot_x,
+			image->hotspot_y);
+		return;
+	}
+
+	if (server->xwayland->cursor) {
+		/*
+		 * The previous configured theme has set the
+		 * default cursor or the xwayland server is
+		 * currently running but still has a cached
+		 * xcursor set that will be used on the next
+		 * xwayland destroy -> lazy startup cycle.
+		 */
+		zfree(server->xwayland->cursor);
+	}
+}
+
 void
 xwayland_server_finish(struct server *server)
 {
